@@ -9,7 +9,7 @@ class State:
     def __init__(
         self,
         state_name: str,
-        state_text: Optional[str] = None,
+        state_text: str = "@@@",
         data_type: Callable = int,
         back_button: Optional[Union[bool, str]] = None,
         **kwargs,
@@ -36,6 +36,7 @@ class State:
         self.long_task = None
         self.refresh_auth = None
         self.operations = None
+        self.dynamic_text = None
 
     def __str__(self):
         return f"State <{self.name}>"
@@ -83,6 +84,12 @@ class State:
         self.regex = regex
         self.regex_error_text = error_message
 
+    @validate_arguments
+    def add_dynamic_text(self, function: Callable):
+        """Add function to build dynamic text regex
+        must return 2 values (regex, message), both can be None"""
+        self.dynamic_text = function
+
     @validate_arguments
     def add_action(self, function: Callable):
         """Add dynamic action"""
